<?php

App::uses('AppComponent', 'Controller/Component');

class AddressManagerComponent extends AppComponent {

	/**
	 * get all the address for a given model / model_id
	 *
	 * @param 	object 	$AddressModel 	The model object to use for loading
	 * @param 	string 	$model  		The model the data belongs to
	 * @param 	int 	$modelId 		The id the address relates to
	 * @return 	array
	 */
	public function getAddresses($AddressModel, $model, $modelId) {
		return $AddressModel->getAddresses($model, $modelId);
	}

	/**
	 * get specific address
	 *
	 * @param 	object 	$AddressModel 	The model object to use for loading
	 * @param 	int 	$addressId 		The id the address
	 * @return 	array
	 */
	public function getAddress($AddressModel, $addressId) {
		return $AddressModel->getAddress($addressId);
	}

	/**
	 * save a new address
	 *
	 * @param 	object 	$AddressModel 	The model object to use for loading
	 * @param 	string 	$model  		The model the data belongs to
	 * @param 	int 	$modelId 		The id the address relates to
	 * @return 	array 	$result 		Two element array - result(bool) / data(array)
	 */
	public function add($AddressModel, $model, $modelId) {
		$addressData = $this->_controller->request->data['Address'];
		$addressData['model'] = $model;
		$addressData['model_id'] = $modelId;

		if ($AddressModel->save(array('Address' => $addressData))) {

			$result = true;
			$returnData = array(
				'message' => 'Address has been added',
				'Address' => array(
					'id' => $AddressModel->id
				)
			);
		} else {

			$validationErrors = array();
			foreach ($AddressModel->validationErrors as $key => $errors) {
				$validationErrors[] = $errors['0'];
			}

			$errors = array(
				'description' => 'Please correct the following issues and try again',
				'list' => $validationErrors
			);

			$result = false;
			$returnData = array(
				'errors' => $errors
			);
		}

		return array(
			'result' => $result,
			'data' => $returnData
		);
	}

	/**
	 * check the given address id belongs to the given user (usually from auth->user())
	 *
	 * @param 	object 	$AddressModel 	The model object to use for loading
	 * @param 	int 	$addressId	 	The address ID to validdate
	 * @param 	int 	$userId 		The user Id to check against
	 * @return 	bool
	 */
	public function addressBelongsToUser($AddressModel, $addressId, $userId) {
		$result = (bool)$AddressModel->find(
			'count',
			array(
				'conditions' => array(
					'Address.id' => $addressId,
					'Address.model' => 'EvCore.User',
					'Address.model_id' => $userId
				),
				'callbacks' => false
			)
		);

		if ($result === true) {
			$returnData = array(
				'message' => 'Address selected'
			);
		} else {
			$returnData = array(
				'errors' => 'The selected address does not belong to the supplied user'
			);
		}

		return array(
			'result' => $result,
			'data' => $returnData
		);
	}

	/**
	 * set a selected address to session
	 *
	 * @param 	string 	$type 		Type of address (billing / devliery)
	 * @param 	int 	$addressId 	Address row ID
	 * @return 	bool
	 */
	public function selectAddress($type, $addressId) {
		return $this->_controller->Session->write(
			'EvCheckout.address.' . $type,
			$addressId
		);
	}

	/**
	 * find selected addresses
	 *
	 * @return 	array
	 */
	public function findAddresses() {
		// if the addresses have been loaded and saved to the template get from there
		if (isset($this->_controller->viewVars['selectedAddresses'])) {
			$addresses = $this->_controller->viewVars['selectedAddresses'];
		} else {
			// read for the session and process each id
			$addressIds = $this->_controller->Session->read('EvCheckout.address');
			$addresses = array();

			foreach ($addressIds as $key => $addressId) {
				$addresses[$key] = $this->getAddress(
					EvClassRegistry::init('EvAddressBook.Address'),
					$addressIds[$key]
				);
			}
		}

		return $addresses;
	}

}
